@@ -1,14 +1,24 @@
 library(nlme)  # needed for fdHess()
+
 # 2004/MAY/04: removing cov2cor (it's there anyway)
 #              changing 'print.coefmat' to 'printCoefmat'
 #              'print.matrix' doesn't work anymore (for strans)
+#
 # 2004/JUL/20: trying to make it fit for CRAN:
 #   removing beta2eba.R (or do you want to explain what it does??)
 #   making summary.eba generic: add '...', rename x to object
-#   changing all 'T' to 'TRUE' (goddamn why??)
+#   changing all 'T' to 'TRUE'
+#
+# 2005/APR/26: added new functions:
+#   pcX            paired-comparison design matrix
+#   wald.test      testing linear hypotheses (Cp = 0) in EBA models
+#   group.test     groupwise testing in EBA models
+#   residuals.eba  deviance and pearson residuals
+#   plot.eba       diagnostic plot
+#   BUG FIX: df in the imbalance test corrected
 
 
-OptiPt = function(M,A=1:I,s=rep(1/J,J)){
+OptiPt <- function(M, A = 1:I, s = rep(1/J, J)){
   # parameter estimation for BTL/Pretree/EBA models
   # M: paired-comparison matrix
   # A: model specification list(c(1,6),c(2,6),c(3,7),...)
@@ -36,6 +46,7 @@ OptiPt = function(M,A=1:I,s=rep(1/J,J)){
   y1 = t(M)[lower.tri(t(M))]  # response vectors
   y0 = M[lower.tri(M)]
   n = y1+y0
+  names(y1) = names(y0) = names(n) = NULL
   logL.sat = sum(dbinom(y1,n,y1/n,log=TRUE))  # likelihood of the sat. model
 
   out = nlm(L,s,y1=y1,m=n,i1=idx1,i0=idx0)  # Newton-type algorithm
@@ -48,6 +59,7 @@ OptiPt = function(M,A=1:I,s=rep(1/J,J)){
 
   fitted = matrix(0,I,I)  # fitted PCM
   fitted[lower.tri(fitted)] = n/(1+idx0%*%p/idx1%*%p)
+  mu <- as.numeric( 1/(1+idx0%*%p/idx1%*%p) )  # predicted probabilities
   fitted = t(fitted)
   fitted[lower.tri(fitted)] = n/(1+idx1%*%p/idx0%*%p)
 
@@ -64,7 +76,7 @@ OptiPt = function(M,A=1:I,s=rep(1/J,J)){
 
   z = list(estimate=p,se=se,ci95=ci,fitted=fitted,logL.eba=logL.eba,
            logL.sat=logL.sat,goodness.of.fit=gof,u.scale=u,
-           hessian=-hes,cov.p=cova,chi.alt=chi.alt,A=A,y1=y1,y0=y0,n=n)
+           hessian=-hes,cov.p=cova,chi.alt=chi.alt,A=A,y1=y1,y0=y0,n=n,mu=mu)
   class(z) = "eba"
   z
 }
@@ -87,18 +99,24 @@ summary.eba = function(object,...){
   dimnames(coef.table) = list(names(coef),c(dn,"z value","Pr(>|z|)"))
 
   tests = rbind(
-    c(1,I*(I-1),sum(dpois(y,mean(y),log=TRUE)),sum(dpois(y,y,log=TRUE)),
-      2*(sum(dpois(y,y,log=TRUE))-sum(dpois(y,mean(y),log=TRUE))),
-      1-pchisq(2*(sum(dpois(y,y,log=TRUE))-sum(dpois(y,mean(y),log=TRUE))),
-        I*(I-1)-1)),
-    c(J-1,I*(I-1)/2,x$logL.eba,x$logL.sat,chi2,pval),
-    c(0,J-1,sum(dbinom(x$y1,x$n,1/2,log=TRUE)),x$logL.eba,
-      2*(x$logL.eba-sum(dbinom(x$y1,x$n,1/2,log=TRUE))),
-      1-pchisq(2*(x$logL.eba-sum(dbinom(x$y1,x$n,1/2,log=TRUE))),J-1)),
-    c(1,I*(I-1)/2,sum(dpois(x$n,mean(x$n),log=TRUE)),sum(dpois(x$n,x$n,log=TRUE)),
-      2*(sum(dpois(x$n,x$n,log=TRUE))-sum(dpois(x$n,mean(x$n),log=TRUE))),
-      1-pchisq(2*(sum(dpois(x$n,x$n,log=TRUE))-sum(dpois(x$n,mean(x$n),log=TRUE))),
-        I*(I-1)-1))
+
+   # mean poisson model vs. saturated poisson model (on y)
+    c(1, I*(I-1), l.1 <- sum(dpois(y,mean(y),log=TRUE)),
+                  l.2 <- sum(dpois(y,y,log=TRUE)),
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,I*(I-1)-1)),
+
+   # EBA model vs. saturated binomial model
+    c(J-1, I*(I-1)/2, x$logL.eba, x$logL.sat, chi2, pval),
+
+   # Null model vs. EBA model
+    c(0, J-1, l.1 <- sum(dbinom(x$y1,x$n,1/2,log=TRUE)), l.2 <- x$logL.eba,
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,J-1)),
+
+   # mean poisson model vs. saturated poisson model (on n)
+    c(1, I*(I-1)/2, l.1 <- sum(dpois(x$n,mean(x$n),log=TRUE)),
+                    l.2 <- sum(dpois(x$n,x$n,log=TRUE)),
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,I*(I-1)/2-1))
+
   )
   rownames(tests) = c('Overall','EBA','Effect','Imbalance')
   colnames(tests) = c('Df1','Df2','logLik1','logLik2','Deviance','Pr(>|Chi|)')
@@ -239,3 +257,158 @@ cov.u = function(eba){
   }
   cov.u
 }
+
+
+pcX = function(nstimuli){
+  # paired comparison design matrix
+  X = matrix(0,choose(nstimuli,2),nstimuli)
+  count = 1
+  for(i in 1:(nstimuli-1)){
+    for(j in (i+1):nstimuli){
+      X[count,i] = 1
+      X[count,j] = -1
+      count = count + 1
+    }
+  }
+  X
+}
+
+
+group.test <- function(groups,A=1:I,s=rep(1/J,J)){
+  # groups: 3d array of group matrices (one matrix per group)
+
+  pool = apply(groups,1:2,sum)  # pooled data matrix
+  I = ncol(pool)  # number of stimuli
+  J = max(unlist(A))  # number of eba parameters
+  ngroups = dim(groups)[3]  # number of groups
+
+  eba.p = OptiPt(pool,A,s)  # EBA for pooled data
+  ebas = NULL  # list of eba models (one per group)
+  for(i in 1:ngroups) ebas[[i]] = OptiPt(groups[,,i],A,s)
+
+  C1 = sum(log(choose(eba.p$n,eba.p$y1)))
+  C2 = 0
+  for(i in 1:ngroups) C2 = C2 + sum(log(choose(ebas[[i]]$n,ebas[[i]]$y1)))
+  C = C1-C2  # combinatorial constant
+
+  logL.eba.group = 0
+  y = NULL; n = NULL
+  for(i in 1:ngroups){
+    logL.eba.group = logL.eba.group + ebas[[i]]$logL.eba
+    y = c(y, c(ebas[[i]]$y1,ebas[[i]]$y0))
+    n = c(n, ebas[[i]]$n)
+  }
+  logL.sat.group = 0
+  for(i in 1:ngroups) logL.sat.group = logL.sat.group + ebas[[i]]$logL.sat
+
+  tests = rbind(
+
+   # mean poisson model vs. saturated poisson group model (on y)
+    c(df1<-1, df2<-ngroups*I*(I-1), l.1 <- sum(dpois(y,mean(y),log=TRUE)),
+      l.2 <- sum(dpois(y,y,log=TRUE)),
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,df2-df1)),
+
+   # EBA group model vs. saturated binomial group model
+    c(df1<-ngroups*(J-1), df2<-ngroups*I*(I-1)/2, l.1 <- logL.eba.group,
+      l.2 <- logL.sat.group,
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,df2-df1)),
+
+   # EBA pool model vs. EBA group model
+    c(df1<-J-1, df2<-ngroups*(J-1), l.1 <- eba.p$logL.eba,
+      l.2 <- logL.eba.group + C,
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,df2-df1)),
+
+   # Null model vs. EBA pool model
+    c(df1<-0, df2<-J-1, l.1 <- sum(dbinom(eba.p$y1,eba.p$n,1/2,log=TRUE)),
+      l.2 <- eba.p$logL.eba,
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,df2-df1)),
+
+   # mean poisson model vs. saturated poisson group model (on n)
+    c(df1<-1, df2<-ngroups*(I*(I-1)/2), l.1 <- sum(dpois(n,mean(n),log=TRUE)),
+                    l.2 <- sum(dpois(n,n,log=TRUE)),
+      dev <- 2*(l.2 - l.1), 1-pchisq(dev,df2-df1))
+
+  )
+  rownames(tests) = c('Overall','EBA.g','Group','Effect','Imbalance')
+  colnames(tests) = c('Df1','Df2','logLik1','logLik2','Deviance','Pr(>|Chi|)')
+
+  z = list(tests=tests)
+  class(z) = "group.test"
+  z
+}
+
+
+print.group.test = function(x,digits=max(3,getOption("digits")-3),na.print="",
+  symbolic.cor=p>4, signif.stars=getOption("show.signif.stars"),...){
+  cat("\nGroup test for EBA models:\n")
+  cat("\n")
+  printCoefmat(x$tests, digits = digits, signif.stars = signif.stars,
+    zap.ind = c(1,2), ...)
+  cat("\n")
+  invisible(x)
+}
+
+
+wald.test <- function(eba,C,u.scale=TRUE){
+  # Wald test of linear hypothesis Cp = 0 for EBA models
+  # u.scale=TRUE: test on the u.scale values
+  # u.scale=FALSE: test on the EBA parameters
+
+  if(u.scale){
+    p = eba$u.scale
+    COV = cov.u(eba)
+  }else{
+    p = eba$estimate
+    COV = eba$cov.p
+  }
+  if(!is.matrix(C)) stop("C is not a matrix.")
+  if(dim(C)[2] != length(p))
+    stop("Column number of C and length of p do not agree.")
+  if(u.scale) colnames(C) = names(eba$u.scale)
+
+  W = t(C%*%p) %*% solve( C%*%COV%*%t(C) ) %*% (C%*%p)
+  z = list(W=W,df=qr(C)$rank,pval=1-pchisq(W,qr(C)$rank),C=C)
+  class(z) = "wald.test"
+  z
+}
+
+
+print.wald.test = function(x, digits = max(3,getOption("digits")-4), ...){
+  cat("\nWald Test: Cp = 0\n\n")
+  cat("C:\n")
+  print(x$C)
+  cat("\nW = ",x$W,", df = ",x$df,", p-value = ",x$pval,"\n",sep='')
+  cat("\n")
+  invisible(x)
+}
+
+
+residuals.eba <- function (object, type = c("deviance", "pearson"), ...){
+
+  dev.resids <- function(y, mu, wt)
+    2 * wt * (y * log(ifelse(y == 0, 1, y/mu)) +
+              (1-y) * log(ifelse(y == 1, 1, (1 - y)/(1 - mu))))
+
+  type <- match.arg(type)
+  wts <- object$n
+  y <- object$y1 / wts
+  mu <- object$mu
+  res <- switch(type,
+    deviance = if(object$goodness['df'] > 0){
+        d.res <- sqrt(pmax(dev.resids(y, mu, wts), 0))
+        ifelse(y > mu, d.res, -d.res)
+      }
+      else rep.int(0, length(mu)),
+    pearson = (y - mu) * sqrt(wts)/sqrt(mu * (1 - mu))
+  )
+  if(!is.null(object$na.action)) res <- naresid(object$na.action, res)
+  res
+}
+
+
+plot.eba <- function(x, ...){
+  plot(x$mu, resid(x), xlab='Predicted choice probabilities',
+       ylab='Deviance residuals')
+  abline(h=0, lty=2)
+  panel.smooth(x$mu, resid(x))
+}
